import click

from pifr.commands.host import get_ssh_host_info, rich_print_hosts
from pifr.commands.ssh import ssh_pull_image


@click.group()
def cli():
    """Pull docker image from remote host."""


@cli.command(name="list")
def list_hosts():
    """列出主机 List hosts"""
    if hosts := get_ssh_host_info():
        rich_print_hosts(hosts)
    else:
        click.echo(click.style("No hosts found in ~/.ssh/config", fg="yellow"))


@cli.command(name="pull")
@click.argument("host", type=str)
@click.argument("image", type=str)
@click.option("--verbose", is_flag=True, help="Verbose output")
def pull_image(host: str, image: str, verbose: bool):
    """在主机上拉取镜像，并导入到本地 Pull image at remote host and save it to local"""
    ssh_pull_image(host, image, verbose)


if __name__ == "__main__":
    cli()



import subprocess
import uuid

import click


def ssh_pull_image(host: str, image: str, verbose: bool = False):
    """Pull docker image at remote host"""

    click.echo(
        click.style("Pulling image ") + click.style(image, fg="cyan") + click.style(" at ") + click.style(host, fg="cyan") + click.style("...")
    )
    _ssh_run(["ssh", host, "docker", "pull", image], "Pulling image done", "Pulling image failed", verbose)

    image_path = f"/tmp/{uuid.uuid4().hex[:7]}.tar.gz"

    click.echo(click.style("Saving image ") + click.style(image, fg="cyan") + click.style(f" to {image_path}..."))
    _ssh_run(["ssh", host, "docker", "save", image, "|", "gzip", ">", image_path], "Saving image done", "Saving image failed", verbose)

    click.echo(click.style("Copying ") + click.style(image_path, fg="cyan") + click.style(f" from {host} to local..."))
    _ssh_run(["scp", f"{host}:{image_path}", image_path], "Copying image done", "Copying image failed", verbose)

    click.echo(click.style("Removing ") + click.style(image_path, fg="cyan") + click.style(" at remote host..."))
    _ssh_run(["ssh", host, "rm", image_path], f"Removing {image_path} success", f"Removing {image_path} error", verbose)

    click.echo(click.style("Loading ") + click.style(image_path, fg="cyan") + click.style(" to local docker..."))
    _ssh_run(["docker", "load", "-i", image_path], "Loading image done", "Loading image failed", verbose)

    click.echo(click.style("Removing ") + click.style(image_path, fg="cyan") + click.style(" at local..."))
    _ssh_run(["rm", image_path], f"Removing {image_path} success", f"Removing {image_path} error", verbose)


def _ssh_run(cmd: list[str], success: str, fail: str, verbose: bool = False):
    result = subprocess.run(cmd, capture_output=True, text=True)
    if result.returncode != 0:
        click.echo(click.style(fail, fg="red"))
        click.echo(click.style(result.stderr, fg="red"))
        exit(result.returncode)
    if verbose and result.stdout:
        click.echo(click.style(result.stdout))
    click.echo(click.style(success, fg="green") + click.style(" ✅\n"))



from dataclasses import dataclass
from pathlib import Path

from rich.console import Console
from rich.table import Table


@dataclass
class HostInfo:
    name: str
    hostname: str = None
    user: str = None
    port: int = None


def get_ssh_host_info() -> list[HostInfo]:
    # 构建 .ssh/config 文件的完整路径
    ssh_config_path = Path.home() / ".ssh" / "config"
    host, hosts = None, []

    def _extract_value(s: str) -> str:
        return s.strip().split(" ", 1)[1].strip()

    if ssh_config_path.exists():
        with ssh_config_path.open("r") as file:
            for line in file:
                line = line.strip()
                if not line or line.startswith("#"):
                    continue
                # init a host data
                if line.startswith("Host "):
                    if host:
                        hosts.append(host)
                    if name := _extract_value(line):
                        host = HostInfo(name)
                    else:  # ifnore error config
                        host = None
                if host and line.startswith("HostName "):
                    host.hostname = _extract_value(line)
                elif host and line.startswith("Port "):
                    host.port = int(_extract_value(line))
                elif host and line.startswith("User "):
                    host.user = _extract_value(line)
        if host:
            hosts.append(host)
    return hosts


def rich_print_hosts(hosts: list[HostInfo]):
    # 创建一个 Console 对象，用于输出表格
    console = Console()
    # 创建一个 Table 对象
    table = Table(show_header=True, header_style="bold")
    # 添加表头
    table.add_column("Host", style="green")
    table.add_column("Hostname", style="yellow")
    table.add_column("Port", style="magenta")
    table.add_column("User", style="cyan")
    # 添加表格行数据
    for host in hosts:
        table.add_row(host.name, host.hostname or "", str(host.port or ""), host.user or "")
    # 使用 Console 对象打印表格
    console.print(table)
